from arkdriver.presentation.presentation import main as presentation
from arkdriver.driver import Driver
from arklibrary.admin import Admin
from pathlib import Path
from arklibrary import Ini
from time import sleep
import requests


URL = 'https://api.loadingproductions.com'
TEST_URL = 'http://127.0.0.1:5000'
TEST_CLOUD_URL = 'http://3.137.160.113'


def ping(url: str):
    count = 0
    while count < 10:
        try:
            res = requests.get(url)
            assert res.status_code == 200, f"Url returned status code: {res.status_code}"
            assert res.json()['connection'] == "successful", "Connection was not successful"
            return
        except Exception as e:
            print(e)
        count += 1
    raise Exception("Unable to get a response from the server: ping")


def get_new_commands(domain: str):
    count = 0
    while count < 10:
        try:
            return requests.get(domain + "/command/new")
        except Exception as e:
            print(e)
        count += 1
    raise Exception("Unable to get a response from the server: get new commands")


def set_command_as_executed(domain: str, command_id: int):
    count = 0
    while count < 10:
        try:
            res = requests.patch(domain + f'/command/{command_id}', json={'executed': True}, headers={'mime': 'application/json'})
            if res.status_code != 200:
                raise Exception("failed to update command as executed [command_id]:", command_id)
            return
        except Exception as e:
            print(e)
        count += 1
    raise Exception("Unable to get a response from the server: update command")


def config():
    path = Path.cwd() / Path('config.ini')
    if not path.exists():
        return
    config = Ini(path)
    if not 'ADMIN' in config:
        return
    admin_password = config['ADMIN']['password']
    admin_player_id = config['ADMIN']['player_id']
    return {'admin_password': admin_password, 'admin_player_id': admin_player_id}


def fetch_commands(url: str, interval=10, wait=5):
    while True:
        res = get_new_commands(url)
        data = []
        try:
            data = [c for c in res.json()]
            print(f"FETCHED: {len(data)}")
        except:
            print(f"[Error] Request to api crashed, response: {res = }")
        if len(data) == 0:
            sleep(wait)
            continue
        i = 0
        while i < len(data):
            print(f"DATA[{i}: {min(i+interval, len(data))}]: {data}")
            yield data[i:i+interval]
            i += interval


def run(url: str, interval=10, wait=5):
    print("Sign into the server.")
    input("Press enter to continue...")
    print()

    print("Create your character and spawn without dying.")
    input("Press enter to continue...")
    print()
    admin_credentials = config()
    if admin_credentials is None:
        password = input("What is the server ADMIN PASSWORD: ")
        while len(password) == 0:
            print("ERROR: The admin password must be longer than 0 characters.")
            password = input("What is the server's ADMIN PASSWORD: ")
        print()

        admin_id = input("What is the ADMIN specimen implant id: ")
        while len(admin_id) != 9 or not admin_id.isnumeric():
            print("ERROR: The specimen implant id must be length 9 and all numbers.")
            admin_id = input("What is the ADMIN specimen implant id: ")
        print()
    else:
        password = admin_credentials['admin_password'].upper()
        admin_id = admin_credentials['admin_player_id']
    print("Close any menu on the screen and make sure your character's inventory is closed.")
    input("Press enter to continue...")

    driver = Driver()
    admin = Admin(driver=driver, password=password, player_id=admin_id)
    admin.enable_admin()
    admin.execute()
    for commands in fetch_commands(url, interval=interval, wait=wait):
        for command in commands:
            admin.command_list.append(command['code'])
            command['executed'] = True
            set_command_as_executed(url, command['id'])
        admin.execute()


def executable():
    ping(URL)
    run(URL, interval=10, wait=5)


def main():
    print("[1] Run driver with official API")
    print("[2] Test driver with local API")
    print("[3] Test driver with cloud API")
    print("[4] presentation")
    response = input("What would you like to run? ")
    choices = ['1', '2', '3', '4']
    while response not in choices:
        print(f"ERROR: your response should be a choice of: {choices}")
        print("[1] Run driver with official API")
        print("[2] Test driver with local API")
        print("[3] Test driver with cloud API")
        print("[4] presentation")
        response = input("What would you like to run? ")

    if response == '1':
        ping(URL + '/ping')
        run(URL, interval=10, wait=5)
    elif response == '2':
        ping(TEST_URL + '/ping')
        run(TEST_URL, interval=10, wait=5)
    elif response == '3':
        ping(TEST_CLOUD_URL + '/ping')
        run(TEST_CLOUD_URL, interval=10, wait=5)
    elif response == '4':
        presentation()


if __name__ == "__main__":
    main()



from arkdriver import Driver
from arklibrary.admin import Admin, Bundle
from arklibrary.blueprints import Island
from time import sleep
from itertools import cycle
import json
import numpy as np
from random import choices
import arkdriver
import os
from pathlib import Path


spectator_ids = []
admin_id = ''
admin: Admin = None
bundle: Bundle = None


def start_admin(driver: Driver, password: str, player_id: str):
    global admin_id
    global admin
    global bundle
    admin_id = player_id
    admin = Admin(driver=driver, password=password, player_id=player_id)
    admin.enable_admin().execute()
    bundle = Bundle(admin)
    return admin


def add_spectator(player_id: str):
    global spectator_ids
    spectator_ids.append(player_id)


def give_spectators_armour():
    armours = [bundle.flak_armour, bundle.desert_armour, bundle.fur_armour, bundle.hazard_armour, bundle.tek_armour]
    bundle.tek_armour(admin_id)
    bundle.execute()
    for spectator, armour in zip(spectator_ids, cycle(armours)):
        armour(spectator, quality=6)
        bundle.execute()


def give_spectators_weapons():
    weapons = [bundle.shotgun_set, bundle.sniper_set, bundle.flame_arrow_set, bundle.compound_set, bundle.rifle_set, bundle.rocket_set]
    bundle.shotgun_set(admin_id)
    bundle.execute()
    for spectator, weapon in zip(spectator_ids, cycle(weapons)):
        weapon(spectator, quality=6)
        bundle.execute()


def give_spectators_fob_set():
    bundle.fob_set(admin_id)
    bundle.execute()
    for spectator in spectator_ids:
        bundle.fob_set(spectator)
        bundle.execute()


def spawn_dinos(blueprint_path, tamed=True, level=150):
    global admin
    global spectator_ids
    for spectator in spectator_ids:
        admin.teleport_to_playerid(spectator)
        admin.spawn_dino(blueprint_path, tamed=tamed, level=level)
    admin.execute()


def teleport_red():
    admin.teleport_red_obelisk()
    for spectator in spectator_ids:
        admin.teleport_player_id_to_me(spectator)
    admin.execute()


def teleport_blue():
    admin.teleport_blue_obelisk()
    for spectator in spectator_ids:
        admin.teleport_player_id_to_me(spectator)
    admin.execute()


def teleport_green():
    admin.teleport_green_obelisk()
    for spectator in spectator_ids:
        admin.teleport_player_id_to_me(spectator)
    admin.execute()



def save_current_coordinates(map, name, num_drops, coords):
    """ Stores the coordinates into file with a name
    :param name: str, name for the coordinates
    """
    data = {"center": {}, "gen1": {}, "gen2": {}, "island": {},
            "scortched": {}, "ragnarok": {}, "extinction": {}, "aberration": {}}
    if map not in data:
        raise NotImplemented("'{}' isn't part of the map list: {}".format(map, data.keys()))
    file_path = Path(os.path.dirname(arkdriver.__file__)) / Path('presentation/saved_coordinates.json')
    if file_path.exists():
        with open(file_path, 'r') as r:
            data = json.load(r)
    if str(num_drops) not in data[map][name]:
        data[map][name][str(num_drops)] = coords
    with open(file_path, "w") as w:
        json.dump(data, w)


def generate_obelisk_drop_coords(admin, map: str, color: str, number_of_drops: int):
    file = Path(os.path.dirname(arkdriver.__file__)) / Path('presentation/saved_coordinates.json')
    with open(file, 'r') as r:
        data = json.load(r)
    coords = None
    name = None
    if color.lower() == 'blue':
        coords = data[map]['blue_obelisk']
        name = 'blue_obelisk_drops'
    elif color.lower() == 'green':
        coords = data[map]['green_obelisk']
        name = 'green_obelisk_drops'
    elif color.lower() == 'red':
        coords = data[map]['red_obelisk']
        name = 'red_obelisk_drops'
    x, y, z, _, _ = coords.values()
    circumference_coords = obelisk_circle_coords([x, y, z], spokes=number_of_drops)
    coordinates = []
    finished = False
    while not finished:
        try:
            coordinates.clear()
            for i, ccc in enumerate(circumference_coords):
                admin.teleport_xyz(*ccc).execute()
                sleep(8)
                admin.driver.copy_coords()
                new_ccc_str = admin.driver.app.get_from_clipboard()
                coordinates.append([float(c) for c in new_ccc_str.split()][:3])
                if len(coordinates) == number_of_drops:
                    finished = True
                elif i + 1 != len(coordinates):
                    break
        except Exception:
            pass
        except KeyboardInterrupt:
            finished = True
    save_current_coordinates(map, name, number_of_drops, coordinates)


def obelisk_circle_coords(obelisk_center_coord, spokes=10):
    x_center, y_center, _ = obelisk_center_coord
    radius = 1763.2620338452252
    offsets = np.linspace(0, 360, spokes)
    coords = []
    for offset in offsets:
        x = radius * np.cos(offset)
        y = radius * np.sin(offset)
        coords.append((x_center + x, y + y_center, 0))
    return coords


def drop_the_supplies(admin, map, quantity=8, quality=0, only=False):
    supply_drops = [
        Island.WHITE_BEACON,
        Island.WHITE_BEACON_DOUBLE_ITEMS,
        Island.GREEN_BEACON,
        Island.GREEN_BEACON_DOUBLE_ITEMS,
        Island.BLUE_BEACON,
        Island.BLUE_BEACON_DOUBLE_ITEMS,
        Island.PURPLE_BEACON,
        Island.PURPLE_BEACON_DOUBLE_ITEMS,
        Island.YELLOW_BEACON,
        Island.YELLOW_BEACON_DOUBLE_ITEMS,
        Island.RED_BEACON,
        Island.RED_BEACON_DOUBLE_ITEMS
    ]
    quality = min(quality, len(supply_drops) - 1)
    drops = supply_drops[quality:]

    file = Path(os.path.dirname(arkdriver.__file__)) / Path('presentation/saved_coordinates.json')
    with open(file, 'r') as r:
        data = json.load(r)
    red = data[map]['red_obelisk_drops'][str(quantity)]
    green = data[map]['green_obelisk_drops'][str(quantity)]
    blue = data[map]['blue_obelisk_drops'][str(quantity)]

    if only:
        for ob_coords in [red, green, blue]:
            for coords in ob_coords:
                x, y, z = coords
                admin.teleport_xyz(x, y, z)
                admin.spawn_beacon(drops[0])
    else:
        weights = (100, 90, 80, 70, 60, 50, 30, 25, 20, 15, 10, 5)[quality:]
        for ob_coords in [red, green, blue]:
            shuffled_drops = choices(drops, weights=weights, k=len(ob_coords))
            for pair in zip(ob_coords, shuffled_drops):
                coords, drop = pair
                x, y, z = coords
                admin.teleport_xyz(x, y, z)
                admin.spawn_beacon(drop)
    admin.execute()


def event_supply_drops(admin, map, quantity=4, quality=4):
    message = ''
    if quality < 4:
        message = "Gamma Supply Drop: Event in {} seconds. Random drops will spawn at every obelisks. PVP is allowed."
    if 4 <= quality < 8:
        message = "Beta Supply Drop: Event in {} seconds. Random drops will spawn at every obelisks. PVP is allowed."
    if 8 <= quality:
        message = "Alpha supply: drop Event in {} seconds. Random drops will spawn at every obelisks. PVP is allowed."
    admin.broadcast(message.format(60))
    admin.execute()
    sleep(30)
    admin.broadcast(message.format(30))
    admin.execute()
    sleep(20)
    admin.broadcast(message.format(5))
    admin.execute()

    drop_the_supplies(admin, map, quantity, quality)



def initial():
    driver = Driver()
    print("Request the map to be Ragnarok.")
    input("Press enter to continue...")
    print()

    print("Request for the ADMIN password.")
    input("Press enter to continue...")
    print()

    print("Join the server and make sure you have a character already created.")
    input("Press enter to continue...")
    print()

    password = input("What is the server's ADMIN PASSWORD: ")
    while len(password) == 0:
        print("ERROR: The admin's server password must be longer than 0 characters.")
        password = input("What is the server's ADMIN PASSWORD: ")
    print()

    admin_id = input("What is the ADMIN specimen implant id: ")
    while len(admin_id) != 9 or not admin_id.isnumeric():
        print("ERROR: The specimen implant id must be length 9 and all numbers.")
        admin_id = input("What is the ADMIN specimen implant id: ")
    print()

    start_admin(driver=driver, password=password, player_id=admin_id)


def present_spectators():
    for i in range(10):
        admin.broadcast("Each player must send Zed3kiah (a.k.a Zed) their specimen implant id in a message. Open your inventory and hover over the diamond. The diamond has a number called your specimen implant id.")
        admin.execute()
        sleep(2)
    spectator = input("What is the SPECTATOR specimen implant id? (Enter 'stop' to proceed): ")
    while spectator != 'stop':
        if len(spectator) != 9 or not spectator.isnumeric():
            print("ERROR: The specimen implant id must be length 9 and all numbers.")
        else:
            spectator_ids.append(spectator)
        spectator = input("What is the SPECTATOR specimen implant id? (Enter 'stop' to proceed): ")
    print()



def present_teleportation():
    for i in range(6):
        admin.broadcast("As an example, I'll teleport you all to each obelisk every 3 seconds. Just to prove a point.")
        admin.execute()
        sleep(2)
    sleep(3)
    teleport_red()
    sleep(3)
    teleport_green()
    sleep(3)
    teleport_blue()
    sleep(3)
    teleport_red()
    sleep(3)
    teleport_green()
    sleep(3)
    teleport_blue()
    sleep(3)


def present_item_giving():
    for i in range(6):
        admin.broadcast("I will now present you with item spawning. Each one of you will receive armour of a different kind.")
        admin.execute()
        sleep(2)
    give_spectators_armour()
    sleep(20)

    for i in range(4):
        admin.broadcast("Each one of you will receive weapons of a different kind.")
        admin.execute()
        sleep(2)
    give_spectators_weapons()
    sleep(20)

    for i in range(4):
        admin.broadcast("Each one of you will receive a fob set.")
        admin.execute()
        sleep(2)
    give_spectators_fob_set()
    sleep(20)


def present_drop_party():
    admin.broadcast("ALERT: A major drop event is about to commence")
    admin.execute()
    sleep(4)

    admin.broadcast("ALERT: Everyone head to an obelisk")
    admin.execute()
    sleep(2)

    admin.broadcast("ALERT: Everyone head to an obelisk")
    admin.execute()
    sleep(2)

    obelisks = [admin.teleport_red_obelisk, admin.teleport_green_obelisk, admin.teleport_blue_obelisk]
    for spectator, obelisk in zip(spectator_ids, cycle(obelisks)):
        obelisk()
        admin.teleport_player_id_to_me(spectator)
    admin.execute()
    drop_the_supplies(admin, 'ragnarok', quantity=8, quality=99, only=False)
    print("\n\n\n\n\n\n\n\n\n")
    print("Please wait until the drop party is finished before you continue.")
    print("\n\n")


def present_ending():
    teleport_blue()

    admin.broadcast("This concludes the presentation")
    admin.execute()
    sleep(5)

    admin.broadcast("I hope you've enjoyed it. Please, now is the time to ask questions.")
    admin.execute()
    sleep(5)



def main():
    initial()
    print()
    input("Press enter to continue...")

    present_spectators()
    print()
    input("Press enter to continue...")

    present_teleportation()
    print()
    input("Press enter to continue...")

    present_item_giving()
    print()
    input("Press enter to continue...")

    present_drop_party()
    print()
    input("Press enter to continue...")

    present_ending()


if __name__ == '__main__':
    main()



"""
This program only controls the basic commands of the program's window.
Like starting, closing, and checking if it exists.
"""

from time import sleep
from pywinauto import keyboard
import win32clipboard
import pickle
from pathlib import Path


class Application:

    def __init__(self):
        self.__desktop = None
        self.__window = None
        self.pane = None
        path = Path(__file__).parent.parent / Path("ark.lnk")
        self.ark_path = path if path.exists() else Path.cwd() / Path("ark.lnk")

    def start(self, resize=False):
        """ Starts the application by running the shortcut file. """
        self.__assert_ark_link_exists()
        from subprocess import Popen
        Popen(str(self.ark_path), shell=True)
        self.__find_desktop_app()
        self.wait_exists()
        if resize:
            sleep(7)
            self.auto_fix_window_position()
            sleep(0.5)
            self.auto_fix_window_size()
            sleep(0.5)
            self.auto_fix_window_position()

    def __find_ark_pywinauto(self, desktop):
        self.__window = desktop.window(title="ARK: Survival Evolved")
        if self.exists():
            self.pane = self.__window['Pane2']
            self.window = self.__window
        else:
            raise LookupError("Application was unable to be located as a background process.")

    def __find_desktop_app(self):
        from pywinauto import Desktop
        self.__desktop = Desktop(backend="uia")
        self.__find_ark_pywinauto(self.__desktop)  # also finds the ark app

    def __assert_ark_link_exists(self):
        if not self.ark_path.exists():
            raise FileNotFoundError(f"'ark.lnk' file is not found in the directory: {self.ark_path}")

    def exists(self, seconds=5):
        """ Checks if the program is currently running. Default times out at 5 seconds. """
        return self.__window.exists(seconds)

    def close(self):
        """ Closes the application """
        if self.exists():
            self.__window.close()

    def wait_exists(self):
        """ halts the program until the ark starts. """
        self.__window.wait('exists')

    def wait_visible(self):
        """ halts the program until the ark is on screen (e.g. not minimized) """
        self.__window.wait('visible')

    def wait_active(self):
        """ Halts the program until ark is the main focus """
        self.__window.wait('active')

    def minimize(self):
        """ Minimizes the window. """
        self.__window.minimize()

    def maximize(self):
        """ Maximizes the window. """
        self.__window.maximize()

    def set_focus(self):
        """
        Makes this window the current focus/active window
        """
        self.__window.set_focus()

    def resize_window(self, width, height):
        """
        Resizes the window to desired width and height.
        :param width: size in pixels
        :param height: size in pixels
        """
        rect = self.__window.rectangle()
        top_left = (rect.left, rect.top)
        bottom_right = (rect.right - 1, rect.bottom - 1)
        self.set_focus()
        self.drag(top_left, (10, 10))
        self.drag(bottom_right, (width - 1, height - 1))
        # TODO: assert window has been resized

    def __get_monitor_size(self):
        """ Returns width, height of monitor resolution """
        from win32api import GetSystemMetrics
        return GetSystemMetrics(0), GetSystemMetrics(1)

    def auto_fix_window_position(self):
        """ Defaults the window position """
        width, height = self.__get_monitor_size()
        self.move_window(width // 2, height // 4)

    def auto_fix_window_size(self):
        """ Defaults the window size """
        rect = self.__window.rectangle()
        top_left = (rect.left + 1, rect.top + 1)
        bottom_right = (rect.right - 1, rect.bottom - 1)
        self.set_focus()
        self.drag(top_left, (10, 10))
        self.drag(bottom_right, (1025 - 1, 613 - 1))

    def __window_pane_offset(self):
        """
        Absolute differences between the whole window and the pane.
        :return: the absolute difference
        """
        window_rect = self.__window.rectangle()
        pane_rect = self.pane.rectangle()
        top = abs(window_rect.top - pane_rect.top)
        left = abs(window_rect.left - pane_rect.left)
        bottom = abs(window_rect.bottom - pane_rect.bottom)
        right = abs(window_rect.right - pane_rect.right)
        return left, top, right, bottom

    def resize_pane(self, width=1016, height=563):
        """
        Resizes the window by desired pane size
        :param width: size in pixels
        :param height: size in pixels
        """
        o_left, o_top, o_right, o_bottom = self.__window_pane_offset()
        self.resize_window(width + o_left + o_right, height + o_top + o_bottom)
        # TODO: assert it has been resized

    def move_window(self, left, top):
        """ Moves the window to a new position as the top left corner being the origin """
        self.set_focus()
        dialog_rect = self.__window['Dialog'].rectangle()
        dialog_left = dialog_rect.left + (dialog_rect.width() // 2)
        dialog_top = dialog_rect.top + (dialog_rect.height() // 2)
        self.drag((dialog_left, dialog_top), (left, top))
        # TODO: assert the window has been moved (e.g. program interruption)

    def has_keyboard_focus(self):
        # will have to use AI to figure this out
        pass

    def double_click_dialog(self):
        self.__window['Dialog'].double_click_input()

    def send_keys(self, string):
        self.set_focus()
        sleep(1)
        for c in string:
            if c == " ":
                keyboard.send_keys("{SPACE down}", pause=0.2)
                keyboard.send_keys("{SPACE up}", pause=0.2)
            elif c == "_":
                # keyboard.send_keys("{VK_SHIFT down}", pause=0.5)
                keyboard.send_keys("_", pause=0.5)
                # keyboard.send_keys("{VK_SHIFT up}", pause=0.5)
            else:
                keyboard.send_keys("{" + "{}".format(c) + " down}", pause=0.2)
                keyboard.send_keys("{" + "{}".format(c) + " up}", pause=0.2)

    def send_key_tab(self):
        keyboard.send_keys("{TAB down}")
        keyboard.send_keys("{TAB up}")

    def send_key_enter(self):
        keyboard.send_keys("{ENTER down}")
        keyboard.send_keys("{ENTER up}")

    def send_key_copy(self):
        keyboard.send_keys("{VK_CONTROL down}", pause=0.5)
        keyboard.send_keys("{c down}", pause=0.5)
        keyboard.send_keys("{VK_CONTROL up}", pause=0.5)
        keyboard.send_keys("{c up}", pause=0.5)

    def send_key_paste(self):
        keyboard.send_keys("{VK_CONTROL down}")
        keyboard.send_keys("{v down}")
        keyboard.send_keys("{VK_CONTROL up}")
        keyboard.send_keys("{v up}")

    def send_key_ctrl_n(self):
        keyboard.send_keys("{VK_CONTROL down}")
        keyboard.send_keys("{n down}")
        keyboard.send_keys("{VK_CONTROL up}")
        keyboard.send_keys("{n up}")

    def save_to_clipboard_text(self, string):
        win32clipboard.OpenClipboard()
        win32clipboard.EmptyClipboard()
        win32clipboard.SetClipboardData(win32clipboard.CF_UNICODETEXT, string)
        # Alternate version if above fails
        # win32clipboard.SetClipboardText(string)
        win32clipboard.CloseClipboard()

    def get_from_clipboard(self):
        win32clipboard.OpenClipboard()
        data = win32clipboard.GetClipboardData()
        win32clipboard.CloseClipboard()
        return data

    def __del__(self):
        """ Close the application when the program finishes """
        # self.close()

    def click(self, coords):
        """ Left click on coordinates """
        self.set_focus()
        self.__window.move_mouse_input(coords=coords)
        self.__window.press_mouse_input(coords=coords)
        self.__window.release_mouse_input(coords=coords)

    def drag(self, start_coords, end_coords):
        """ Click and drag from starting coordinates to ending coordinates """
        self.set_focus()
        self.__window.press_mouse_input(coords=start_coords)
        self.__window.release_mouse_input(coords=end_coords)

    def sides(self, pywin_obj):
        """ Gets the left, top, right, and bottom coordinates of a window/pane """
        rect = pywin_obj.rectangle()
        return rect.left, rect.top, rect.right, rect.bottom

    def window_size(self):
        rect = self.window.rectangle()
        return rect.width(), rect.height

    def pane_size(self):
        rect = self.pane.rectangle()
        return rect.width(), rect.height()

    def window_wrapper(self):
        return self.__window.wrapper_object()

    def pane_wrapper(self):
        return self.pane.wrapper_object()

    def __cached_index(self):
        index = 1
        path = Path('images/menus/cached/index.pickle')
        if path.exists():
            with open(path, 'rb') as r:
                index = pickle.load(r)
        with open(path, 'wb') as w:
            pickle.dump(index + 1, w)
        return index

    def absolute_pane_coords(self, pane_coords):
        left, top, right, bottom = self.sides(self.pane)
        return left + pane_coords[0], top + pane_coords[1]


def __test_startup():
    ark = Application()
    ark.start()
    ark.wait_exists()
    ark.set_focus()
    ark.close()


def __test_resizing():
    ark = Application()
    ark.start()
    ark.wait_exists()
    ark.set_focus()

    ark.resize_window(433, 999)
    ark.resize_pane(1000, 600)

    if ark.pane.width != 1000 or ark.pane.height != 600:
        raise ArithmeticError("Width or height is inaccurate")

    ark.close()


def main():
    __test_startup()
    __test_resizing()


if __name__ == "__main__":
    main()



import arkdriver

def test_imports():
    arkdriver.main.__testing__ = True
    try:
        from arkdriver import run
    except Exception:
        print()
        assert False, arkdriver.console.error("Cannot import run from arkdriver.")
    arkdriver.main.__testing__ = False


def test_run():
    arkdriver.main.__testing__ = True
    from arkdriver import run
    try:
        run()
    except Exception as e:
        print()
        assert False, arkdriver.console.error(f"Function run() from main failed: {e}")
    arkdriver.main.__testing__ = False

